import re

import numpy as np
import pandas as pd

from .types import RaceInfo, RaceMetadata

QUESTION_PATTERN = re.compile(r"(?P<question>.*) \[(?P<option>.*)\]")


def parse_rows(rows):
    rows = np.asarray(rows)
    race_infos = []
    for metadata, slice_ in parse_header(rows[0]):
        data = rows[1:, slice_].astype(int)
        ballots_df = pd.DataFrame(data, columns=metadata.names)
        ballots_size = ballots_df.groupby(metadata.names).size()
        ballots = [list(e) for e in ballots_size.index]
        votes = ballots_size.tolist()
        race_info = RaceInfo(metadata, ballots, votes)
        race_infos.append(race_info)
    return race_infos


def parse_header(header):
    current_question = None
    current_options = []

    questions = []
    options = []
    starts = []
    ends = []
    for col_idx, col in enumerate(header):
        match = QUESTION_PATTERN.match(col)
        if match:
            question = match.group("question")
            option = match.group("option")
            if question != current_question:
                if current_question is not None:
                    ends.append(col_idx)
                    options.append(current_options)
                    current_options = []
                questions.append(question)
                starts.append(col_idx)
                current_question = question
            current_options.append(option)
        else:
            if current_question is not None:
                current_question = None
                ends.append(col_idx)
                options.append(current_options)
                current_options = []

    if current_question is not None:
        ends.append(col_idx + 1)
        options.append(current_options)

    num_questions = len(questions)
    assert len(options) == num_questions, options
    assert len(starts) == num_questions, starts
    assert len(ends) == num_questions, ends

    return [
        (RaceMetadata(q, 1, o), slice(s, e))
        for q, o, s, e in zip(questions, options, starts, ends)
    ]
